/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 *
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function(){
dojo.provide("jazz.app.ResourceLink");

dojo.require("dojo.i18n");
dojo.require("dijit._Widget");
dojo.require("dijit._Templated");

dojo.require("jazz.app.proxy");
dojo.require("jazz.ui.HoverPopup");
dojo.require("jazz.app.internal.loginUtil");
dojo.require("jazz.app.ua.internal.util");
dojo.require("jazz.app.internal.ResourceLinkAggregator");
dojo.require("jazz.util.html");

dojo.requireLocalization("jazz.app.internal", "ResourceLinkMessages");

var HoverPopup = jazz.ui.HoverPopup;
var TYPE_PREVIEW = "http://jazz.net/identifiers/presentations/preview";
var CSS_ARROW = "arrow";
var CSS_RL = "jazz-app-ResourceLink";

var expandedResourceLink;

// use a single event handler for the web page since we'll have possibly many
// navigation links in a single page
dojo.connect(dojo.body(), "onclick", function(e){
	var target = e && e.target;
    if (!target) {
    	return; // early return
    }
	if (expandedResourceLink && !dojo.isDescendant(target, jazz.ui._getMasterPopup().domNode)) {
		expandedResourceLink.close();
	}
    if (target.tagName !== "A" || target.__jazzType !== "ResourceLink") {
    	return;
	}
    var widget = dijit.byId(target.getAttribute("widgetId"));
    if (widget) {
    	widget["onClick"](e);
    }
});

dojo.declare("jazz.app.ResourceLink", dijit._Widget, {
    
	// summary:
	//		A hyperlink referencing a Jazz resource. We take the URL of the
	//		link's HREF attribute to determine the resource (Work Item, Plan,
	//		etc.) and then send back information to display a summary of the
	//		resource when the user hovers over the link. The summaries presented
	//		can have a small and large view depending on how much content the
	//		user wishes to see.
	//
	// parameters:
	//		uri: (String) The uri that will be used to find the available
	//				presentations. REQUIRED if the given linkNode does not have
	//				a uri attached to the HREF attribute.
	//		(optional) presentationUri: (String) A URL that will replace the
	//				link's HREF attribute, used to send a user to a different
	//				presentation of the given resource.
	//		(optional) label: (String) A string or HTML snippet that will be
	//				placed as the child node/inner text of the link/anchor node
	//				the Resource Link will be attached to.  If no srcNodeRef is 
	//				given, this argument is REQUIRED.
	//		(optional) srcNodeRef: (Anchor Node) The anchor node in the page
	//				that the Resource Link will be attached to. NOTE: if the uri
	//				parameter is not given, the href value of the given	node
	//				will be used in its place as the resource to reference.
	//
	// usage:
	//		var navLink = new jazz.ui.ResourceLink({}, linkNode);
	//		-or-
    //		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//                  label: "Label for the Anchor Node"
	//				});
	//		-or-
	//		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//				},
	//				linkNode);
	//		-or-
	//		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//					presentationUri: "anotherUrl"
	//				},
	//				linkNode);
    //		-or-
	//		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//					presentationUri: "anotherUrl",
    //                  label: "New Label for the Anchor Node"
	//				},
	//				linkNode);
    //		-or-
	//		var navLink = new jazz.ui.ResourceLink(
	//				{
	//					uri: "aResourceuri", 
	//					presentationUri: "anotherUrl",
    //                  label: "Label for the Anchor Node"
	//				});
	// since:
	//		0.7
	create: function(args, srcNodeRef){
		this._retainLinkText = args.retainLinkText;
		if ( !srcNodeRef && args.uri )
		{
			this.linkNode = document.createElement("A");
			this.uri = args.uri;
			
			if (args.presentationUri) 
				this.linkNode.href = args.presentationUri;
			else 
				this.linkNode.href = this.uri;
					
			this.linkNode.innerHTML = jazz.util.html.removeScript(args.label);
		}
		else if (srcNodeRef && srcNodeRef.tagName === "A" && !args.uri) {
			this.linkNode = srcNodeRef;
			this.uri = this.linkNode.href;
			
			if (args.presentationUri) 
				this.linkNode.href = args.presentationUri;
			
			if ( args.label )
				this.linkNode.innerHTML = jazz.util.html.removeScript(args.label);
		}
		else if (srcNodeRef && srcNodeRef.tagName === "A" && args.uri) {
			this.linkNode = srcNodeRef;
			this.uri = args.uri;
			
			if (args.presentationUri) 
				this.linkNode.href = args.presentationUri;
			else 
				this.linkNode.href = this.uri;
				
			if ( args.label )
				this.linkNode.innerHTML = jazz.util.html.removeScript(args.label);
		}
		
		if (this.linkNode) {
			this.linkNode.__jazzType = "ResourceLink";
			dojo.addClass(this.linkNode, CSS_RL);
			this.inherited("create", arguments, [args, this.linkNode]);
			this._registerWithAggregator();
			//Style the link
			var arrow = document.createElement("img");
			dojo.addClass(arrow, CSS_ARROW);
			arrow.src = dojo.moduleUrl("jazz.app", "internal/images/ResourceLink/rh-twistie.gif");
			this.linkNode.appendChild(arrow);
		}
		else
			throw new Error("ResourceLink REQUIRES that either the uri parameter or a valid reference to an Anchor Node is given");
	},
	
	_registerWithAggregator: function () {
		// Register this ResourceLink with the ResourceLinkAggregator.  The aggregator will retrieve
		// data for all ResourceLinks when the page is finished loading
		var aggregator = jazz.app.internal.resourceLinkAggreagtor();
		aggregator.register(this.uri, dojo.hitch(this, this._createPresentation), dojo.hitch(this, this._errorPresentation), false);
	},

	_createPresentation: function(results) {
		
		if (results.oauthRequired){
			this.oauthRequired = true;
			this.token = results.token;
			
			//Create the HoverPopup
			this._popupLauncher = new HoverPopup({
				around: this.domNode,
				createContents: dojo.hitch(this, this._initPopup),
				onOpen: dojo.hitch(this, this._openPopup),
				onClose: dojo.hitch(this, this._closePopup),
				customClass: "jazz-app-ResourceHover"
			});
		
			return;
		} else {
			this.oauthRequired = false;
		}		
		this.showHover = false;
		this.showExpand = false;

		if (results.title) {
			this.heading = jazz.util.html.removeScript(results.title);
			if (!this._retainLinkText) {
				this.linkNode.innerHTML = this.heading;
			}
		}

		if (results.icon) 
			this.icon = results.icon;
		
		if (results.smallPreview) {
			this.smallHoverUrl = results.smallPreview;
			this.showHover = true;
		}
		
		if (results.largePreview) {
			this.largeHoverUrl = results.largePreview;
				
			if (this.showHover) 
				this.showExpand = true;
			else 
				this.showHover = true;
		}
		
		if (results.smallWidth && (Math.ceil(results.smallWidth) == Math.floor(results.smallWidth)) ) 
			this.smallWidth = results.smallWidth;
		
		if (results.smallHeight && (Math.ceil(results.smallHeight) == Math.floor(results.smallHeight)) ) 
			this.smallHeight = results.smallHeight;
		
		if (results.largeWidth && (Math.ceil(results.largeWidth) == Math.floor(results.largeWidth)) ) 
			this.largeWidth = results.largeWidth;
		
		if (results.largeHeight && (Math.ceil(results.largeHeight) == Math.floor(results.largeHeight)) ) 
			this.largeHeight = results.largeHeight;
		
		//Create the HoverPopup
		if (this._popupLauncher) {
			this._popupLauncher.destroy();
			this._popupLauncher = null;
		}
		this._popupLauncher = new HoverPopup({
			around: this.domNode,
			createContents: dojo.hitch(this, this._initPopup),
			onOpen: dojo.hitch(this, this._openPopup),
			onClose: dojo.hitch(this, this._closePopup),
			customClass: "jazz-app-ResourceHover"
		});
	},
	
	_errorPresentation: function (err) {
		console.warn("ResourceLink Error (" + this.uri + "): " + err.message);
		this._disableHover();
	},
	
	_disableHover: function () {
		if (this._popupLauncher) {
			this._popupLauncher.destroy();
		}
		for (var i=0; i < this.linkNode.childNodes.length; i++) {
			if (this.linkNode.childNodes[i].tagName == "IMG" && this.linkNode.childNodes[i].className == CSS_ARROW) {
				this.linkNode.removeChild(this.linkNode.childNodes[i]);
				break;
			}
		}
		dojo.removeClass(this.linkNode, CSS_RL);
	},
	
	_initPopup: function(hoverPopup){
		
		if( this.oauthRequired ){
			this._linkMetadata = new net.jazz.app._LinkMetadata({
				smallHoverUrl: null,
				largeHoverUrl: null,
				icon: null,
				heading: null,
				showExpand: false,
				largeWidth: null,
				largeHeight: null,
				smallWidth: null,
				smallHeight: null,
				_popupLauncher: this._popupLauncher,
				linkNode: this.linkNode,
				uri: this.uri,
				token: this.token,
				oauthRequired: this.oauthRequired				
			});
		
			return this._linkMetadata.domNode;
		}

		this._linkMetadata = new net.jazz.app._LinkMetadata({
			smallHoverUrl: this.smallHoverUrl,
			largeHoverUrl: this.largeHoverUrl,
			icon: this.icon,
			heading: this.heading,
			showExpand: this.showExpand,
			largeWidth: this.largeWidth,
			largeHeight: this.largeHeight,
			smallWidth: this.smallWidth,
			smallHeight: this.smallHeight,
			_popupLauncher: this._popupLauncher,
			linkNode: this.linkNode,
			uri: this.uri,
			oauthRequired: false
		});

		return this._linkMetadata.domNode;
	},
	
	_openPopup: function(){
		this._linkMetadata.open();
	},
	
	_closePopup: function() {
		this._popupLauncher.programmatic = false;
		if (this.showExpand) {
			this._linkMetadata._expandBtnDiv.className = "expansion";
		}
		expandedResourceLink = null;
	}
});

//Creates te DOM elements that will hold the preview and presentation links displayed in the Hover Popup
dojo.declare("net.jazz.app._LinkMetadata", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("jazz.app", "internal/templates/ResourceLink.html"),
	
	constructor: function () {
		this.messages = dojo.i18n.getLocalization("jazz.app.internal", "ResourceLinkMessages");
	},
	
	postCreate: function(){
		dojo.body().appendChild(this.domNode);
		this.inherited(arguments);
		var loginLink = jazz.app.internal.loginUtil.insertLoginLink(this.oauthLoginDiv);
		this.connect(loginLink, "onclick", this.oauthLogin);
		
		this._closeButton.removeChild(this._closeButton.firstChild);
		this._expandButton.removeChild(this._expandButton.firstChild);
		
		if(this.oauthRequired){
			dojo.removeClass(this.oauthLoginDiv, "hideLogin");
		}
		else{
			this._renderJsonPresentation();	
		}
	},
	
	/**
	* Toggles between displaying the small and large summary views.
	* @param {Object} e - mouse event
	*/
	displaySummary: function(e){
		dojo.stopEvent(e);
			
		if (this.showExpand) {
			if (!this._isFullDisplay) {
				this.displayLargeSummary();
				this._isFullDisplay = true;
				this._popupLauncher.programmatic = "block";
				this._expandBtnDiv.className = "noExpansion";
			}
			
			this._popupLauncher.position(this.linkNode);
		}
	},
	
	/**
	* Display the large content area.
	*/
	displayLargeSummary: function(){
		this._isFullDisplay = true;
		if (this.showExpand) 
			this._expandButton.className = "minimizeSummary";
			
		if (this.largeHoverUrl) {
			if (this.largeWidth) {
				this.hoverIframe.style.width = this.largeWidth + "px";
				if (dojo.isIE) {
					// IE needs explicit width on the divider and width 100% breaks it out the side
					// allow 4 px for margins on wrapper dix
					this.divider.style.width = (this.largeWidth - 4) + "px";
				}
			}
			if (this.largeHeight) {
				this.hoverIframe.style.height = this.largeHeight + "px";
			}
			this.hoverIframe.src = this.largeHoverUrl;
		}
		expandedResourceLink = this._popupLauncher;
		// yield the call to updateEllipsis to ensure the UI is painted
		setTimeout(dojo.hitch(this,this.updateEllipsis),0);
	},
	
	/**
	* Display the small content area.
	*/
	displaySmallSummary: function(){
		this._isFullDisplay = false;
		if (this.showExpand) 
			this._expandButton.className = "expandSummary";
		if (this.smallHoverUrl) {
			if (this.smallWidth) {
				this.hoverIframe.style.width = this.smallWidth + "px";
				if (dojo.isIE) {
					// IE needs explicit width on the divider and width 100% breaks it out the side
					// allow 4 px for margins on wrapper dix
					this.divider.style.width = (this.smallWidth - 4) + "px";
				}
			}
			if (this.smallHeight) {
				this.hoverIframe.style.height = this.smallHeight + "px";
			}
			this.hoverIframe.src = this.smallHoverUrl;
		}
		expandedResourceLink = null;
		// yield the call to updateEllipsis to ensure the UI is painted
		setTimeout(dojo.hitch(this,this.updateEllipsis),0);
	},
	
	updateEllipsis: function() {
		// hide the ellipsis first in case there is just enough room without it
		dojo.addClass(this.ellipsis, "hidden");
		// check if the titleNode is taller than the titleBar
		// subtract half the titlebar height in comparison to account for difference in single line sizes
		if (this.titleNode.offsetHeight - (this.titleBar.offsetHeight/2) > this.titleBar.offsetHeight) {
			// if the title node is too tall, it is wrapping, so show the ellipsis
			dojo.removeClass(this.ellipsis, "hidden");
			this.titleNode.title = this.tooltipText;
		} else {
			this.titleNode.title = "";
		}
	},
	
	/**
	* Method that determines if the available presentations should be shown or not in the Hover Popup when it appears.
	*/
	open: function(){
		if (this.smallHoverUrl) 
			this.displaySmallSummary();
		else 
			this.displayLargeSummary();
	},
	
	/**
	* Close the hover popup.
	* @param {Object} e - a mouse event.
	*/
	close: function(e){
		dojo.stopEvent(e);
		this._popupLauncher.close();
	},
	
	/**
	 * Access the compact rendering document on an outside server by logging in with
	 * Oauth.
	 * @param {Object} evt - mouse event
	 */
	oauthLogin: function(evt){
		dojo.stopEvent(evt);
		// handleAs "xml" doesn't work if the content-type isn't "text/xml"
		// In this case, the content type should be "application/x-jazz-compact-rendering"
		// so we handleAs "text" and parse the XML document in the handler
		var dfd = jazz.client.xhrPost({
			url: this.uri,
			headers: {
				"Accept": "application/x-jazz-compact-rendering"
			},
			handleAs: "text"
		});
		dfd.addCallback(dojo.hitch(this, this.renderOauthResponse));
		dfd.addErrback(dojo.hitch(this, this.renderOauthError));
	},

	/**
	 * This method breaks down the values in the JSON object acquired from the CompactRendering servlet
	 * that are then passed to properties of the _LinkMetaData class.
	 */
	_renderJsonPresentation: function(){
		this.updateTitle(this.heading);
		
		if (this.smallHoverUrl) {
			this._smallHoverHeadDfd = jazz.client.xhrHead({
				url: this.smallHoverUrl
			});
			this._smallHoverHeadDfd.addCallback(dojo.hitch(this, function (xhr) {
				if (xhr.status >= 400) {
					console.warn("Response code " + xhr.status + " for small hover (" + this.smallHoverUrl + ")");
					this.smallHoverUrl = dojo.moduleUrl("jazz.app", "internal/html/SmallPreviewMissing.html");
				} 
				if (!this._isFullDisplay) {
					this.hoverIframe.src = this.smallHoverUrl;
				}
			}, this._smallHoverHeadDfd.ioArgs.xhr));
		}
		if (this.largeHoverUrl) {
			this._largeHoverHeadDfd = jazz.client.xhrHead({
				url: this.largeHoverUrl
			});
			this._largeHoverHeadDfd.addCallback(dojo.hitch(this, function (xhr) {
				if (xhr.status >= 400) {
					console.warn("Response code " + xhr.status + " for large hover (" + this.largeHoverUrl + ")");
					this.largeHoverUrl = dojo.moduleUrl("jazz.app", "internal/html/LargePreviewMissing.html");
				}
				if (this._isFullDisplay) {
					this.hoverIframe.src = this.largeHoverUrl;
				}
			}, this._largeHoverHeadDfd.ioArgs.xhr));
		}
		this.setHeightsAndWidths();
			
		//IE does not like <img> without a source value, so we hide the image unless there is a src value
		if (this.icon) {
			this.iconImg.src = this.icon;
			this.iconImg.className = "icon";
		}
		
		if (this.showExpand) {
			this._isFullDisplay = false;
			this._expandBtnDiv.className = "expansion";
		}
	},
	
	/**
	 * When authenticating against OAuth, we receive back the compact redering document in XML.
	 * This method is used to parse that document and then have the ResourceLinkAggregator resend
	 * an xhr request for the other ResourceLinks that access the same server.
	 * @param {Object} results
	 */
	renderOauthResponse: function(results) {
		results = dojox.data.dom.createDocument(results);
		var compactRendering = results.getElementsByTagName("jp:CompactRendering");
		
		var title = results.getElementsByTagName("dc:title");
		if (title){
			this.updateTitle(jazz.util.html.removeScript(title[0].childNodes[0].nodeValue));//PLACE TITLE STRING
		}
		
		var icon = results.getElementsByTagName("jp:icon");
		if(icon){
			this.iconImg.src = icon[0].attributes.getNamedItem("rdf:resource").value
			this.iconImg.className = "icon";
		}
		
		var smallHover = results.getElementsByTagName("jp:smallPreview");
		
		if(smallHover){
			this.smallHoverUrl = smallHover[0].attributes.getNamedItem("rdf:resource").value;
			this.smallWidth= smallHover[0].attributes.getNamedItem("preferred_width").value;
			this.smallHeight = smallHover[0].attributes.getNamedItem("preferred_height").value;
			
			if(this.smallHoverUrl)
				this.showExpand == true;
				
			this._smallHoverHeadDfd = jazz.client.xhrHead({
				url: this.smallHoverUrl
			});
			this._smallHoverHeadDfd.addCallback(dojo.hitch(this, function (xhr) {
				if (xhr.status >= 400) {
					console.warn("Response code " + xhr.status + " for small hover (" + this.smallHoverUrl + ")");
					this.smallHoverUrl = dojo.moduleUrl("jazz.app", "internal/html/SmallPreviewMissing.html");
				}
				if (!this._isFullDisplay) {
					this.hoverIframe.src = this.smallHoverUrl;
				}
			}, this._smallHoverHeadDfd.ioArgs.xhr));
		}
		
		var largeHover= results.getElementsByTagName("jp:largePreview");
		
		if(largeHover){
			this.largeHoverUrl = largeHover[0].attributes.getNamedItem("rdf:resource").value;
			this.largeWidth= largeHover[0].attributes.getNamedItem("preferred_width").value;
			this.largeHeight = largeHover[0].attributes.getNamedItem("preferred_height").value;
			
			if(!this.largeHoverUrl)
				this.showExpand = false;
			
			this._largeHoverHeadDfd = jazz.client.xhrHead({
				url: this.largeHoverUrl
			});
			this._largeHoverHeadDfd.addCallback(dojo.hitch(this, function (xhr) {
				if (xhr.status >= 400) {
					console.warn("Response code " + xhr.status + " for large hover (" + this.largeHoverUrl + ")");
					this.largeHoverUrl = dojo.moduleUrl("jazz.app", "internal/html/LargePreviewMissing.html");
				}
				if (this._isFullDisplay) {
					this.hoverIframe.src = this.largeHoverUrl;
				}
			}, this._largeHoverHeadDfd.ioArgs.xhr));
		}
		this.setHeightsAndWidths();
		
		dojo.addClass(this.oauthLoginDiv, "hideLogin");
		
		if (this.showExpand) {
			this._isFullDisplay = false;
			this._expandBtnDiv.className = "expansion";
		}
		
		var aggregator = jazz.app.internal.resourceLinkAggreagtor();
		aggregator.resendOauth(this.token);
	},
	
	updateTitle: function(heading) {
		var index;
		var title = [];
		var tooltip = [];
		while ((index = heading.indexOf("<")) != -1) {
			var str = heading.substring(0, index);
			if (str.length > 0) {
				tooltip.push(str);
				title.push(str.split("").join("<wbr/>").replace(/ <wbr\/>/g,"&nbsp;"));
			}
			var endIndex = heading.indexOf(">");
			title.push(heading.substring(index, endIndex + 1));
			heading = heading.substring(endIndex + 1);
		}
		if (heading && heading.length > 0) {
			tooltip.push(heading);
			title.push(heading.split("").join("<wbr/>").replace(/ <wbr\/>/g,"&nbsp;"));
		}
		this.titleNode.innerHTML = title.join("");
		this.tooltipText = tooltip.join("");
	},
	
	setHeightsAndWidths: function () {
		if (this.smallWidth && this.smallWidth > 500) {
			this._popupLauncher.maxWidth = this.smallWidth;
		}
		if (this.smallHeight && this.smallHeight + 50 > 300) {
			this._popupLauncher.maxHeight = this.smallHeight + 50;
		}
		if (this.largeWidth && this.largeWidth > 500 && (this.largeWidth > this.smallWidth || !this.smallWidth)) {
			this._popupLauncher.maxWidth = this.largeWidth;
		}
		if (this.largeHeight && this.largeHeight + 50 > 300 && (this.largeHeight > this.smallHeight || !this.smallHeight)) {
			this._popupLauncher.maxHeight = this.largeHeight + 50;
		}
	}
});

})();