/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.LicenseInfoWidget");
dojo.require("com.ibm.team.repository.web.client.internal.licensing"); //$NON-NLS-1$
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler"); //$NON-NLS-1$
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal", "LicenseInfoWidgetMessages");

var messages;

var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var getExpirationInfo = com.ibm.team.repository.web.client.internal.licensing.getExpirationInfo;
var getLicenseInfo = com.ibm.team.repository.web.client.internal.licensing.getLicenseInfo;

dojo.declare("com.ibm.team.repository.web.ui.internal.LicenseInfoWidget", [dijit._Widget, dijit._Templated], {
	
	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/templates/LicenseInfoWidget.html"),
	
	postCreate: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal", "LicenseInfoWidgetMessages");
		
		dojo.subscribe(com.ibm.team.repository.web.internal.events.licenseChange, this, "_showLicenseInfo");
		
		var self = this;
		setTimeout(function() {
			getLicenseInfo(new ServiceResponseHandler(self, "_showLicenseInfo", "_showLicenseInfoErr"));
		}, 4000);		
	},
	_showLicenseInfo: function(licenseInfo) {
		var disabledReason = dojo.getObject("serverLicense.disabledReason", false, licenseInfo || "");
		var expirationTime = dojo.getObject("serverLicense.expirationTime", false, licenseInfo || "");
		
		// if there's no expiration time, don't show anything
		if (!expirationTime && !disabledReason) {
			this.expirationInfo.innerHTML = "";
			this.expirationWrapper.style.display = "none";
		} else if(disabledReason) {
			this.expirationInfo.className = "expirationWarning";
			this.expirationInfo.innerHTML = "<a href='" + licenseInfo.serverLicense.buyURL + "' target='_blank'>" + disabledReason + "</a>";
		} else {
			var expirationInfo = getExpirationInfo(expirationTime);
			var purchaseLink = "<a href='" + licenseInfo.serverLicense.buyURL + "' target='_blank'>";
			this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.trialLicense, [ purchaseLink, "</a> "]);
			if(expirationInfo.isExpired || expirationInfo.daysRemaining <= 14) {
				this.expirationInfo.className = "expirationWarning";
			} else {
				this.expirationInfo.className = "expirationInfo";
			}
			if(expirationInfo.isExpired) {
				this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.hasExpired, [ this.expirationInfo.innerHTML ]);
			} else if (expirationInfo.daysRemaining === 0) {
				this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.expiresToday, [ this.expirationInfo.innerHTML ]);
			} else if (expirationInfo.daysRemaining === 1) {
				this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.expiresInOneDay, [ this.expirationInfo.innerHTML ]);
			} else {
				var daysLeft = expirationInfo.daysRemaining;
				this.expirationInfo.innerHTML = dojo.string.substitute(this.messages.expiresInDays, [ this.expirationInfo.innerHTML, daysLeft ]);
			}
			this.expirationWrapper.style.display = "inline";
		}
	},
	_showLicenseInfoErr: function(errorObj) {
		console.warn(dojo.string.substitute(this.messages.noLicenseInfo, [ errorObj.message ]));
	}
	
});

var EVENTS_ROOT = "com.ibm.team.repository.web.internal.events";
dojo.setObject(EVENTS_ROOT, {
	licenseChange: EVENTS_ROOT + ".licensesChanged"
});

})();