/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2008-2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.CrossServerCommunication");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStepHeader");
dojo.require("com.ibm.team.repository.web.ui.internal.wizard.WizardForm");
dojo.require("com.ibm.team.repository.web.client.internal.OAuthAdminClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");


dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");

(function() {
var PACKAGE_PREFIX = "com.ibm.team.repository.web.ui.internal.admin.server.";
var messages;
var WizardStepHeader = com.ibm.team.repository.web.ui.wizard.WizardStepHeader;
var WizardForm = com.ibm.team.repository.web.ui.internal.wizard.WizardForm;
var OAuthAdminClient = com.ibm.team.repository.web.client.internal.OAuthAdminClient;
var SRH = com.ibm.team.repository.web.transport.ServiceResponseHandler;

var REMOVE_ICON_SRC = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/remove.gif");
var ROOTSERVICES = "/rootservices"

dojo.declare(PACKAGE_PREFIX + "CrossServerCommunication", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/SetupOAuth.html"),
	
	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
		this.stepTitle = messages.stepTitle; 
	},
	
	postCreate: function () {
		this.header =  new WizardStepHeader({stepTitle: this.stepTitle, reset: dojo.hitch(this,"reload")});
		this.stepHeaderArea.appendChild(this.header.domNode);
		
		var constructor = dojo.getObject(PACKAGE_PREFIX + "OAuthRequest");
		var args = {
			formButton: this.requestButton,
			setStatusCallback: dojo.hitch(this, this.setFormStatus), 
			templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/wizard/templates/SimpleWizardForm.html")
		};
		this.oauthForm = new constructor(args);
		this.oauthRequestArea.appendChild(this.oauthForm.domNode);
		this.reload();
	}, 
	
	reload: function () {
		this.header.clearMessages();
		dojo.toggleClass(this.provisionalStepArea, "noDisplay", true);
		this.oauthForm.clearData();
		this.reloadFriends();
	},
	
	reloadFriends: function () {
		dojo.toggleClass(this.noFriends, "noDisplay", true);
		dojo.toggleClass(this.friendsTable, "noDisplay", true);
		dojox.data.dom.removeChildren(this.friendsTableBody);	
		this.getFriends();
	},
		
	errorCallback: function (errorObj, contextMsg) {
		this.header.displayServiceCallbackErrorMessage(errorObj, contextMsg);
	},
		
	acquiredProvisionalKey: function(key) {
		this.provisionalKey = key.key;
		this.storeProvisionalKey();				

		var newKey = document.createElement("div");
		newKey.className = "spaced-all";
		var needsAuth = dojo.string.substitute(this.messages.keyNeedsAuthorization, [this.provisionalKey]);
		newKey.innerHTML = needsAuth;
		
		var newLink = document.createElement("div");
		newLink.className = "spaced-all border";
		var server = this.oauthForm.getServer();
		var serverLink = "<a href='" + server + "/admin#action=com.ibm.team.repository.admin.configureOAuth' target=\"_blank\">";
		var needsAuth = dojo.string.substitute(this.messages.authorizationLink, [server, serverLink, "</a>"]);
		newLink.innerHTML = needsAuth;
		
		this.provisionalContentArea.appendChild(newKey);
		this.provisionalContentArea.appendChild(newLink);
		dojo.toggleClass(this.provisionalStepArea, "noDisplay", false);
	}, 
	
	errorRequestingKey: function (errorObj) {
		var serverUrl = this.oauthForm.getData().url;
		var contextMsg = dojo.string.substitute(this.messages.requestKeyError, [serverUrl]);
		this.errorCallback(errorObj, contextMsg)
	},
	
	storeProvisionalKey: function() {
		var data = this.oauthForm.getData();
		var args = {
			title: data.title, 
        	oauthDomain: data.url,
        	rootServices: data.rootServices,
        	oauthConsumerKey: this.provisionalKey,
        	oauthConsumerSecret: data.consumerSecret
		};
		OAuthAdminClient.postNewFriend(new SRH(this, "successStoredProvisionalKey", "errorPostingFriend"), args);	
	},
	
	successStoredProvisionalKey: function () {
		this.header.clearMessages();
		this.reloadFriends();
	},
	
	errorPostingFriend: function (errorObj) {
		this.errorCallback(errorObj, this.messages.postFriendError);
	},
	
	setFormStatus: function (status) {
		if (status === -1) {
			this.header.clearMessages();
		}
		else {
			this.header.showStatus({
				summary: status.message,
				type: status.type
			});
		}
	}, 
	
	onRequestButtonClick: function () {
		this.requestButton.disabled = true;
		this.header.showStatus({summary: this.messages.requestingAccess, type: WizardStepHeader.STATUS_TYPES.INFO});
		this.oauthForm.runAction(this);
	}, 
	
	getFriends: function () {
		OAuthAdminClient.getFriends(new SRH(this, "displayFriends", "errorFetchingFriends"));			
	},
	
	displayFriends: function (friends) {
		dojo.toggleClass(this.friendsArea, "noDisplay", false);
		
		if (friends){
			this.friendList = friends;
			if(this.friendList.length > 1) {
				this.sortFriends();
			}
			var self = this;
			dojo.forEach(this.friendList, function(friend){
                self.addFriendListRow(friend);
            });
		}
		
		dojo.toggleClass(this.noFriends, "noDisplay", friends);
		dojo.toggleClass(this.friendsTable, "noDisplay", !friends);
	},
	
	errorFetchingFriends: function (errorObj) {
		this.errorCallback(errorObj, this.messages.fetchFriendsError);
	},
	
	sortFriends: function () {	
		return this.friendList.sort(function(a, b){
			var aTitle = a.title.toLowerCase();
			var bTitle = b.title.toLowerCase();
			return aTitle > bTitle ? 1 : -1;
		});
	},
	
	addFriendListRow: function(friend) {
		var row = document.createElement("tr");
		var deleteParam = friend.oAuthConsumerKey;
		row.setAttribute("id","row-" + deleteParam);
		
		var cell1 = document.createElement("td");
		cell1.innerHTML = friend.title;
		row.appendChild(cell1);
		
		var cell2 = document.createElement("td");
		cell2.innerHTML = friend.rootServicesURL;
		row.appendChild(cell2);
		
		var cell3 = document.createElement("td");
		cell3.innerHTML = friend.oAuthDomain;
		row.appendChild(cell3);
		
		var cell4 = document.createElement("td");
		cell4.innerHTML = friend.oAuthConsumerKey;
		row.appendChild(cell4);
			
		var cell5 = document.createElement("td");
		
		//Build delete button
        var del = document.createElement("img");
        del.setAttribute('src', REMOVE_ICON_SRC);
		del.setAttribute('id', "del-" + deleteParam);
		del.setAttribute('title', this.messages.removeFriendButton);
        
		cell5.appendChild(del);
		row.appendChild(cell5);
		
		this.friendsTableBody.appendChild(row);
		dojo.connect(del,"onclick",this, "_deleteFriend");
		dojo.addClass(del,"delButton");
	}, 
	
	_deleteFriend: function(e) {
		var verifyDelete = confirm(this.messages.confirmDelete);
        if (verifyDelete) {
			var key = e.target.id;
			key = key.substring(4); 
			this._keyPendingDeletion = key;
			var friend = this._getFriendForKey(key);
			if (friend) {
				var title = friend.title;
				OAuthAdminClient.deleteFriend(new SRH(this, "removeFriendEntry", "errorDeletingFriend"), {title: title});
			}
		}
	},
	
	_getFriendForKey: function(key) {
		if (this.friendList) {
			for (friend in this.friendList) {
				if (this.friendList[friend].oAuthConsumerKey === key) {
					return this.friendList[friend];
				}
			}
		}
		return null;
	},
	
	removeFriendEntry: function() {
		var elem = dojo.byId("row-" + this._keyPendingDeletion);
		this.friendsTableBody.removeChild(elem);
		if (this.friendsTableBody.childNodes.length == 0) {
			dojo.toggleClass(this.noFriends, "noDisplay", false);
			dojo.toggleClass(this.friendsTable, "noDisplay", true);
		}
	}, 
	
	errorDeletingFriend: function (errorObj) {
		var contextMsg = dojo.string.substitute(this.messages.deleteFriendError, [this._titlePendingDeletion]);
		this.errorCallback(errorObj, contextMsg);
	}
	
});

var FORM_HEADER_DEF = [
	{type: "label", labelKey: "propertyLabel"},
	{type: "data", labelKey: "valueLabel"},
	{type: "description", labelKey: "descriptionLabel"}
];
var OAUTH_REQUEST_TABLE = {
	header: FORM_HEADER_DEF,
	rows: [
		{name: "title", labelKey: "titleLabel", descriptionKey: "titleDescrption"},
		{name: "url", labelKey: "serverUriLabel", descriptionKey: "serverUriDescrption"},
		{name: "rootServices", labelKey: "rootServicesLabel", descriptionKey: "rootServicesDescrption"},
		{name: "consumerSecret", labelKey: "secretLabel", type: "password", descriptionKey: "secretDescrption"},
		{name: "repeatSecret", labelKey: "secretLabel2", type: "password", descriptionKey: "secretDescrption2", mustMatch: {
			matchName: "consumerSecret",
			errorMessageKey: "secretMismatch"
		}},
		{name: "trusted", labelKey: "trustedLabel", type: "checkbox", descriptionKey: "trustedDescription"}
	]
};

dojo.declare(PACKAGE_PREFIX + "OAuthRequest", WizardForm, {
	
	constructor: function() {
		this.messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
		this.heading =  "";
		this.step = "";
		this.description = "";
	},
	
	postCreate: function () {
		this.inherited(arguments);
		this.lastUrl = this.getServer();
		dojo.connect(this,"handleChange", this, "helpRootServicesInput");
		this.validate();
	},
	
	tableContent: OAUTH_REQUEST_TABLE,
		
	runAction: function(caller) {
		var valueMap = this.getData();
		var args = new Object();
		for (i in valueMap) {
			if (i == "repeatSecret") {
				continue;
			}
			args[i] = valueMap[i];
		}
		args.url = args.url + "/oauth-request-consumer";
		OAuthAdminClient.requestRemoteProvisionalKey(new SRH(caller, "acquiredProvisionalKey", "errorRequestingKey"), args);
	},
	
	getServer: function (){
		var valueMap = this.getData();
		for (i in valueMap) {
			if (i == "url") {
				return valueMap[i];
			}
		}
		return -1
	},
	
	helpRootServicesInput: function () {
		if (this.lastUrl != this.getServer()) {
			//get data field for url and rootServices
			for(var i=0; i<this.dataFields.length; i++){
				var name = this.dataFields[i].name;
				if (name == "url") {
					var urlDataField = this.dataFields[i];
				} else if (name == "rootServices") {
					var rootServicesDataField = this.dataFields[i];
				}  
			}
			if (urlDataField && urlDataField.value && urlDataField.value.length > 0) {
				if(rootServicesDataField && !rootServicesDataField.value) {
					//set rootservices + URL
					rootServicesDataField.value = urlDataField.value + ROOTSERVICES;
				}
			} 
		} 
		this.lastUrl = this.getServer();
	},
	
	validate: function() {
		var status = this.inherited("validate", arguments);
		if (!status.isValid) {
			status.message = this.messages.correctFields;
			this.setStatusCallback(status);
		} else {
			this.setStatusCallback(-1);
		}
		this.formButton.disabled = !status.overrideNext;
		
		return status;
	}
	
});


})();