/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2005, 2009. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {
dojo.provide("com.ibm.team.repository.web.ui.internal.admin.server.OAuthView");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dijit.TitlePane");
dojo.require("dojox.data.dom");

dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.team.repository.web.client.internal.OAuthAdminClient");
dojo.require("com.ibm.team.repository.web.ui.wizard.WizardStepHeader");

dojo.require("dojo.i18n");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "OAuthViewMessages");
dojo.requireLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");

var messages;
var setupMessages;
var OAuthAdminClient = com.ibm.team.repository.web.client.internal.OAuthAdminClient;
var ServiceResponseHandler = com.ibm.team.repository.web.transport.ServiceResponseHandler;
var WizardStepHeader = com.ibm.team.repository.web.ui.wizard.WizardStepHeader;
var STATUS_TYPES = WizardStepHeader.STATUS_TYPES;
var PENDING = "pending";

dojo.declare("com.ibm.team.repository.web.ui.internal.admin.server.OAuthView", [dijit._Widget, dijit._Templated], {

	templatePath: dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/server/templates/OAuthView.html"),

	constructor: function() {
		this.messages = messages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.admin.server", "OAuthViewMessages");
		this.setupMessages = setupMessages = dojo.i18n.getLocalization("com.ibm.team.repository.web.ui.internal.wizard", "SetupOAuthStepMessages");
	},	
	
	postCreate: function() {
		this.header = new WizardStepHeader({ 
			stepTitle: this.messages.heading,
			reset: dojo.hitch(this,"refresh")
		});
		this.headerArea.appendChild(this.header.domNode);
		
		dojo.connect(this._registerConsumerButton, "onclick", this, "_registerConsumerHandler");
		this.connect(this._consumerNameInput,"onkeyup","handleKeyUp");
		this.connect(this._consumerNameInput,"onchange","validate");
		this.connect(this._consumerSecretInputAgain,"onkeyup","handleKeyUp");
		this.connect(this._consumerSecretInputAgain,"onchange","validate");
		
		this.refresh();
		
	},
	
	refresh: function() {
		this._getConsumerList();
		this._getProvisionalRegistrations();
		this._consumerSecretInputAgain.value = "";
		this._consumerSecretInput.value = "";
		this._consumerNameInput.value ="";
		this._registerConsumerButton.disabled = true;
		this.header.clearMessages();
	},
	
	_registerConsumerHandler: function () {
		this._registerConsumerButton.disabled = true;
		var consumerSecret = this._consumerSecretInput.value;
		var consumerName = this._consumerNameInput.value;
		
		var srh = new ServiceResponseHandler(this, "_handleConsumerRegistrationSuccess", "displayServiceCallbackErrorMessage");
		OAuthAdminClient.registerNewConsumer(srh, consumerName,  consumerSecret);
		return false;
	},
	
	_handleConsumerRegistrationSuccess: function (consumerKey) {
		
		this._consumerSecretInputAgain.value = "";
		this._consumerSecretInput.value = "";
		this.header.showStatus({ 
			type: STATUS_TYPES.OK,
			summary: dojo.string.substitute(this.messages.successMessage, [consumerKey])
		});

		// Do the save in two steps, because it requires 
		// less privledges to save a consumer than it 
		// does to make a consumer trusted
		if (this._consumerTrustedInput.checked) {
			var srh = new ServiceResponseHandler(this, "_handleConsumerTrustSuccess", "displayServiceCallbackErrorMessage");
			OAuthAdminClient.setConsumerTrusted(srh, consumerKey, true);		
		} else {
			this._getConsumerList();
		}
		
	},
	
	_handleConsumerTrustSuccess: function () {
		this._getConsumerList();
	},
	
	_getConsumerList: function () {
		dojox.data.dom.removeChildren(this.consumersTableBody);	
		dojo.toggleClass(this.noConsumer, "noDisplay", true);
		var srh = new ServiceResponseHandler(this, "displayConsumersList", "displayServiceCallbackErrorMessage");
		OAuthAdminClient.getConsumers(srh);
	},
	
	_deleteConsumer: function(e) {
		var verifyDelete = confirm(this.messages.confirmDelete);
        if (verifyDelete) {
			var consumerKey = e.target.id;
			consumerKey = consumerKey.substring(4);
			this._keyPendingDeletion = consumerKey;
			var srh = new ServiceResponseHandler(this, "removeConsumerEntry", "displayServiceCallbackErrorMessage");
			OAuthAdminClient.deleteConsumer(srh, consumerKey);
		}
	},
	
	removeConsumerEntry: function() {
		var elem = dojo.byId("row-" + this._keyPendingDeletion);
		this.consumersTableBody.removeChild(elem);
		
	},
	
	sortConsumers: function () {	
		return this.consumerList.sort(function(a, b){
			var aName = a.name.toLowerCase();
			var bName = b.name.toLowerCase();
			return aName > bName ? 1 : -1;
		});
	},
	
	displayConsumersList: function (consumers) {
		if (consumers){
			this.consumerList = consumers;
			if(this.consumerList.length > 1) {
				this.sortConsumers();
			}
			var self = this;
			dojo.forEach(this.consumerList, function(consumer){
                self.addConsumerListRow(consumer);
            });
		}
		dojo.toggleClass(this.consumerTable, "noDisplay", !consumers);
		dojo.toggleClass(this.noConsumer, "noDisplay", consumers);
	},
	
	addConsumerListRow: function(consumer) {
		var row = document.createElement("tr");
		row.setAttribute("id","row-" + consumer.key);
		
		var nameCell = document.createElement("td");
		var name = document.createTextNode(consumer.name);
		nameCell.appendChild(name);
		row.appendChild(nameCell);
		this._consumerNameInput.value = "";
		
		var cell2 = document.createElement("td");
		cell2.innerHTML = consumer.key;
		row.appendChild(cell2);
		
		var cell3 = document.createElement("td");
		cell3.innerHTML = consumer.trusted ? this.messages.trustedValue : this.messages.untrustedValue;
		row.appendChild(cell3);
		this._consumerTrustedInput.checked = false;
			
		var cell4 = document.createElement("td");
		
		//Build delete button
        var del = document.createElement("img");
        var removeIconSrc = dojo.moduleUrl("com.ibm.team.repository.web", "ui/internal/admin/users/images/remove.gif");
        del.setAttribute('src', removeIconSrc);
		del.setAttribute('id', "del-" + consumer.key);
		del.setAttribute('title', this.messages.removeConsumerButton);
        
		cell4.appendChild(del);
		row.appendChild(cell4);
		
		this.consumersTableBody.appendChild(row);
		dojo.connect(del,"onclick",this, "_deleteConsumer");
		dojo.addClass(del,"delButton");
	},
	
	_getProvisionalRegistrations: function () {
		// clear table & no provisional msg
		dojox.data.dom.removeChildren(this.provisionalTableBody);
		dojo.toggleClass(this.noProvisional, "noDisplay", true);
		
		var srh = new ServiceResponseHandler(this, "displayProvisionalRegistrations", "displayServiceCallbackErrorMessage");
		OAuthAdminClient.getProvisionalRegistrations(srh);
	},
	
	displayProvisionalRegistrations: function (provisional) {
		if (provisional){
			this.provisionalKeys = provisional;
			var self = this;
			dojo.forEach(this.provisionalKeys, function(key){
                self.addProvisionalRegistrationRow(key);
            });
		}
		dojo.toggleClass(this.provisionalTable, "noDisplay", !provisional);
		dojo.toggleClass(this.noProvisional, "noDisplay", provisional);
		dojo.toggleClass(this.provisionalSaveButton, "noDisplay", !provisional);
	},
	
	addProvisionalRegistrationRow: function(provisionalKey) {
		var row = document.createElement("tr");

		var nameCell = document.createElement("td");
		var name = document.createElement("input");
		name.value = provisionalKey.name;
		name.type = "text";
		name.setAttribute("id","name-" + provisionalKey.key);
		nameCell.appendChild(name);
		row.appendChild(nameCell);
		provisionalKey._nameCell = nameCell;
		
		var cell1 = document.createElement("td");
		var key = document.createTextNode(provisionalKey.key);
		cell1.appendChild(key);
		row.appendChild(cell1);
		
		var cell2 = document.createElement("td");
		var trusted = document.createTextNode(provisionalKey.trusted ? messages.trustedValue : messages.untrustedValue);
		cell2.appendChild(trusted);
		row.appendChild(cell2);
		
		var cell3 = document.createElement("td");
		var grantTrusted = document.createElement("input");
		grantTrusted.type = "checkbox";
		grantTrusted.checked = provisionalKey.trusted;
		provisionalKey._grantTrusted = grantTrusted;
		cell3.appendChild(grantTrusted);
		row.appendChild(cell3);
		provisionalKey._cell3 = cell3;
		
		var cell4 = document.createElement("td");
		var select = this.createAcceptSelect(provisionalKey);
		provisionalKey._acceptSelect = select;
		dojo.connect(provisionalKey._acceptSelect, "onchange", this, "handleProvisionalChange");
		cell4.appendChild(select);
		row.appendChild(cell4);
		provisionalKey._cell4 = cell4;

		this.provisionalTableBody.appendChild(row);
	},
	
	createAcceptSelect: function (provisionalKey) {
		var node = document.createElement("select");
		var option = document.createElement("option");
		option.text = this.messages.pendingLabel;
		option.value = PENDING;
		node.options[node.options.length] = option;
		option = document.createElement("option");
		option.text = this.messages.approveLabel;
		option.value = "true";
		node.options[node.options.length] = option;
		option = document.createElement("option");
		option.text = this.messages.rejectLabel;
		option.value = "false";
		node.options[node.options.length] = option;
		return node;
	},
	
	handleProvisionalChange: function () {
		var dirty = false;
		for (var i = 0; i < this.provisionalKeys.length; i++) {
			if (this.provisionalKeys[i]._acceptSelect && this.provisionalKeys[i]._acceptSelect.value != PENDING) {
				dirty = true;
				break;
			}
		}
		this.provisionalSaveButton.disabled = !dirty;
	},		
			
	saveProvisional: function () {
		var self = this;
		this.provisionalSaveButton.disabled = true;
		var openRequests = 0;
		dojo.forEach(this.provisionalKeys, function(provisionalKey){   
			if (provisionalKey._acceptSelect && provisionalKey._acceptSelect.value != PENDING) {
				provisionalKey.grant = provisionalKey._grantTrusted.checked
				var acceptProvisional = {
					key: provisionalKey,
					
					displayAcceptedRegistration: function(response){
						openRequests--;
					
						var trust = this.key.grant ? self.messages.trustedValue : self.messages.untrustedValue;
						this.key._cell3.replaceChild(document.createTextNode(trust), this.key._cell3.firstChild);
						this.key._grantTrusted = null;
						
						var approval = provisionalKey._acceptSelect.value === "true" ? self.messages.approvedLabel : self.messages.rejectedLabel;
						this.key._cell4.replaceChild(document.createTextNode(approval), this.key._cell4.firstChild);
						this.key._acceptSelect = null;	
						
						var name = provisionalKey.name;
						this.key._nameCell.replaceChild(document.createTextNode(name), this.key._nameCell.firstChild);
						if(openRequests == 0) {
							self._getConsumerList();
						}
						
					},
					handleAcceptError: function(err){
						self.displayServiceCallbackErrorMessage(err);
					}
				}
				provisionalKey.name = dojo.byId("name-" + provisionalKey.key).value;
				openRequests++;
				var srh = new ServiceResponseHandler(acceptProvisional, "displayAcceptedRegistration", "handleAcceptError");
				OAuthAdminClient.acceptProvisionalRegistration(srh, provisionalKey.name, provisionalKey.key, provisionalKey._grantTrusted.checked, provisionalKey._acceptSelect.value);
			}		
		});
		
	},
	
	displayServiceCallbackErrorMessage: function(errorObj) {
		this.header.displayServiceCallbackErrorMessage(errorObj);
	},
	
	clearInlineError: function(elem) {
		dojo.removeClass(elem,"inlineError");
		elem.innerHTML = "";
	},
	
	validate: function() {
		
		clearTimeout(this.onKeyUpTimout);
		
		if(this._consumerSecretInput.value != this._consumerSecretInputAgain.value || this._consumerNameInput.value == "") {
			//this.setupMessages.secretMismatch
			if(this._consumerNameInput.value == "") {
				dojo.addClass(this.nameErr,"inlineError");
				this.nameErr.innerHTML = this.messages.fieldNotBlank;
			} else {
				this.clearInlineError(this.nameErr);
			}
			if(this._consumerSecretInput.value != this._consumerSecretInputAgain.value) {
				dojo.addClass(this.mismatchErr,"inlineError");
				this.mismatchErr.innerHTML = this.setupMessages.secretMismatch;
			} else {
				this.clearInlineError(this.mismatchErr);
			}
			this.header.showStatus({ 
				type: STATUS_TYPES.ERROR,
				summary: this.setupMessages.correctFields
			});
			this._registerConsumerButton.disabled = true;
		} else {
			this._registerConsumerButton.disabled = false;
			this.clearInlineError(this.mismatchErr);
			this.clearInlineError(this.nameErr);
			this.header.clearMessages();
		}

	},
	
	handleKeyUp: function(e) {
		if(e.target && e.target.tagName === "INPUT") {
			if(this.onKeyUpTimout){
				clearTimeout(this.onKeyUpTimout);
			}
			var f = dojo.hitch(this, this.validate);
			var timeout = 2000;
			if (e.target._jazzRow  && e.target._jazzRow.hasError) {
				timeout = 500;
			}
			this.onKeyUpTimout = setTimeout(f, timeout);
		}
	}

});

})();